# -*- coding: utf-8 -*-

#nombre de valeurs à trier
n = 10000

#générer un vecteur de valeurs aléatoires
import numpy
numpy.random.seed(0)
alea = numpy.random.random(size=n)

#librairie graphique
import matplotlib.pyplot as plt
plt.plot(alea)

#algorithme de tri par insertion
#https://fr.wikipedia.org/wiki/Tri_par_insertion
def tri_insertion(tab):
    #faire une copie
    vec = numpy.copy(tab)
    #parcourir le vecteur
    for i in range(1,len(vec)):
        #mémorisation
        x = vec[i]
        #décalage
        j = i
        while (j > 0) and (vec[j-1] > x):
            vec[j] = vec[j-1]
            j = j - 1
        #insertion
        vec[j] = x
    #retourner le vecteur trié
    return vec

#trier
import time
start = time.perf_counter()
alea_tri = tri_insertion(alea)
end = time.perf_counter()
print("Duree = ", end-start," secondes")

#vérification
plt.plot(alea_tri)

#JIT pour fonction
from numba import jit

@jit(nopython=True)
def tri_insertion_jit(tab):
    #faire une copie
    vec = numpy.copy(tab)
    #parcourir le vecteur
    for i in range(1,len(vec)):
        #mémorisation
        x = vec[i]
        #décalage
        j = i
        while (j > 0) and (vec[j-1] > x):
            vec[j] = vec[j-1]
            j = j - 1
        #insertion
        vec[j] = x
    #retourner le vecteur trié
    return vec

#mesurer de nouveau 
start = time.perf_counter()
alea_tri_jit = tri_insertion_jit(alea)
end = time.perf_counter()
print("Duree = ", end-start," secondes")

#vérification
print(numpy.sum(alea_tri == alea_tri_jit))

#regression simple avec descente du gradient
@jit(nopython=True, parallel=True)
def gradient_regression():       
    #paramètre alpha algo gradient
    alpha = 0.005
    iterations = 1000
    #génération des données
    n_obs = 2000000
    numpy.random.seed(0)
    #matrice X
    X = numpy.full(shape=(n_obs,2),fill_value = 1.0)
    X[:,0] = numpy.random.random(size=n_obs)
    #vecteur y = 2 * x + 5 + bruit aléatoire
    y = 2.0 * X[:,0] + 5.0 + numpy.random.normal(loc=0,scale=0.05,size=n_obs)
    #vecteur des coefs à estimer - valeurs de départ
    coef = numpy.array([0.5,0.5])
    #vecteur des scr
    scr = numpy.zeros(iterations)
    #commencer les itérations
    for i in range(iterations):
        #prédictions sur la base des coef. estimés
        yPred = numpy.dot(X,coef)
        #erreur de prédiction
        err = y-yPred
        #somme des carrés des résidus
        scr[i] = numpy.sum(numpy.square(err))
        #vecteur gradient
        gradient = -2.0 / n_obs * numpy.dot(numpy.transpose(X),err)
        #calcul de la nouvelle version des paramètres
        coef = coef - alpha * numpy.transpose(gradient)
        #fin des itérations
    return scr,coef

#lancer
start = time.perf_counter()
estimation_reg = gradient_regression()
end = time.perf_counter()
print("Duree = ", end-start," secondes")

#coefficients estimés
print("Coef. estimés : ",estimation_reg[1])
